﻿using System;
using System.Globalization;
using System.Linq;

using Platform.Net;

namespace Platform.Geo
{
    public class GeoPolygon : GeoShape
    {
        public GeoPosition[][][] Points { get; set; }

        /// <summary>
        /// Convert to WKT
        /// </summary>
        /// <returns></returns>
        public override string ToWKT()
        {
            // Multi Polygon
            if( Points.Length > 1 )
                return string.Format("MULTIPOLYGON({0})", string.Join(",",
                    Points.Select(m => string.Format("({0})", string.Join(", ",
                        m.Select(l => string.Format("({0})", string.Join(", ",
                            l.Select(p => string.Format(CultureInfo.InvariantCulture, "{0} {1}", p.Longitude, p.Latitude))))))))));

            // Single Polygon
            return string.Format("POLYGON({0})", string.Join(",",
                Points[0].Select(l => string.Format("({0})", string.Join(", ",
                    l.Select(p => string.Format(CultureInfo.InvariantCulture, "{0} {1}", p.Longitude, p.Latitude)))))));
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPolygon FromWKT(string wkt)
        {
            if (string.IsNullOrEmpty(wkt)) return null;
            return FromWKT(new NetStringReader(wkt));
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPolygon FromWKT(NetStringReader reader)
        {
            // create reader
            reader.Skip();

            // check polygon types
            switch (reader.ReadPattern("POLYGON", "MULTIPOLYGON"))
            {
                case 0:
                    return new GeoPolygon { Points = new GeoPosition[][][] { ReadWktMultiLineData(reader) } };
                case 1:
                    return new GeoPolygon { Points = ReadWktMultiPolygonData(reader) };
                default:
                    throw new FormatException("Missing literal 'POLYGON' or 'MULTIPOLYGON' in wkt ");
            }
        }

        /// <summary>
        /// Returns true if polygon is clockwise
        /// </summary>
        public bool IsClockwise( int polygon, int ring )
        {
            double sum = 0;
            var points = Points[polygon][ring];
            int count = points.Length;

            for (int i = 0; i < points.Length; i++)
                sum += (points[(i + 1) % count].Longitude - points[i].Longitude) * (points[(i + 1) % count].Latitude + points[i].Latitude);

            return sum >= 0;
        }

        /// <summary>
        /// Get bounds Of all
        /// </summary>
        /// <returns></returns>
        public override GeoRectangle GetBounds()
        {
            GeoRectangle bounds = null;
            foreach (var polygon in Points)
                foreach (var points in polygon)
                    foreach (var point in points)
                        if (bounds == null) bounds = new GeoRectangle { NorthwestCorner = point, SoutheastCorner = point };
                        else bounds = bounds.Union(point);
            return bounds;
        }
    }
}
